@@ -1,8 +1,8 @@
-import jsonpickle
 import functools
+import cPickle
+import jsonpickle
 
 import minsol
-import cPickle
 
 
 class Acceptor(minsol.Object):
@@ -188,7 +188,40 @@ class Client(minsol.Object):
 
     def _fetch_call(self, raw_request):
         """
-        @return: raw_response
+        :return: raw_request
+        :rtype: str
+        """
+        raise NotImplementedError
+
+
+class AsyncClient(Client):
+    def _rpc_call(self, method_name, callback, *args, **kwargs):
+        args = list(args)
+
+        request = self.request_class(
+            ident=self.last_id, method_name=method_name,
+            args=args, kwargs=kwargs)
+
+        self.last_id += 1
+
+        raw_request = request.as_raw()
+
+        def _fetched(raw_response):
+            response = self.response_class.get_from_raw(request, raw_response)
+
+            result = response.result
+            if isinstance(result, Exception):
+                callback(None, result)
+                return
+
+            callback(result, None)
+
+        self._fetch_call(_fetched, raw_request)
+
+    def _fetch_call(self, callback, raw_request):
+        """
+        :param callback: ``def callback(result, error)``
+        :rtype: function
         """
         raise NotImplementedError
 
