@@ -164,7 +164,6 @@ class RMQServer(minsol.Object):
             self.stop()
 
     def stop(self):
-        self.channel.basic_cancel()
         self.channel.close()
 
     def init_broker(self):
@@ -209,8 +208,7 @@ class RMQServer(minsol.Object):
         self.channel.basic_publish(
             exchange='',
             routing_key=reply_to,
-            body=raw_response,
-            mandatory=True)
+            body=raw_response)
 
 
 class RMQClient(hens.Client):
@@ -266,6 +264,70 @@ class JSONRMQClient(RMQClient, hens.JSONClient):
     pass
 
 
+class RMQAsyncServer(minsol.Object):
+    """
+    RabbitMQ asyncRPC server
+    """
+    request_class = hens.Request
+    response_class = hens.Response
+
+    def init(self, acceptor_class, connection, queue_name, **other):
+        self.acceptor_class = acceptor_class
+        self.connection = connection
+        self.queue_name = queue_name
+
+    def start(self):
+        self.connection.add_on_open_callback(self.connection_opened)
+
+    def stop(self):
+        self.channel.close()
+
+    def connection_opened(self, connection):
+        self.connection.channel(self.channel_opened)
+
+    def channel_opened(self, channel):
+        self.channel = channel
+
+        self.channel.queue_declare(
+            self.queue_declared,
+            queue=self.queue_name,
+            durable=True, auto_delete=False)
+
+    def queue_declared(self, frame):
+        self.channel.basic_consume(
+            self.request_accepted,
+            queue=self.queue_name, no_ack=True)
+
+    def request_accepted(self, channel, frame, properties, body):
+        raw_request = body
+        acceptor = self.acceptor_class()
+        request = None
+
+        try:
+            request = self.request_class.get_from_raw(raw_request)
+
+            callback = functools.partial(
+                self.executed,
+                properties.reply_to, request)
+
+            request.execute(acceptor, callback)
+
+        except hens.Error as error:
+            self.executed(properties.reply_to, request, error)
+
+    def executed(self, reply_to, request, result):
+        if not reply_to:
+            return
+
+        response = self.response_class(request=request, result=result)
+
+        raw_response = response.as_raw()
+        self.channel.basic_publish(
+            exchange='',
+            routing_key=reply_to,
+            body=raw_response)
+
+
 class RMQAsyncClient(hens.AsyncClient):
     def init(self, connection, queue_name,
             response_queue_pool_size=10,
@@ -346,12 +408,9 @@ class RMQAsyncClient(hens.AsyncClient):
     def _fetch_call(self, callback, raw_request):
 
         def _response_getted(channel, frame, props, body):
-            #print channel, frame, props, body
-
             callback(body)
             self.return_response_queue(frame.routing_key)
 
-
         def _response_queue_getted(rqueue_name):
             self.response_callbacks[rqueue_name] = _response_getted
 
@@ -389,5 +448,10 @@ class RMQAsyncClient(hens.AsyncClient):
         self.response_queues[rqueue_name] = rchannel
 
 
+class JSONRMQAsyncServer(RMQAsyncServer):
+    request_class = hens.JSONRequest
+    response_class = hens.JSONResponse
+
+
 class JSONRMQAsyncClient(RMQAsyncClient, hens.JSONClient):
     pass
